! -*- F90 -*- 
     subroutine normbf(nshlf,ipst)
BEGIN_DOC
! normalize given expansion coefficients in a contraction 
!  G = x^l sum_i d_i N(a_i,l) Exp(-a_i r 2)
!  the normalization factors N(a_i,l) are 
! Sqrt[Sqrt[(2 a / pi )^3] (4 a)^l/Factorial2[2 l - 1]]
!
! the overlap integral between two normalized primitives of same l
! and at the same center gives 
! S(a_i,a_j,l)= Sqrt[(2a_i)^(l+3/2)] 
!              Sqrt[(2a_j)^(l+3/2)]/(a_i+a_j)^(l+3/2)
!
! to normalize the function we divide by 
! Sqrt[ sum_i sum_j d_i d_j S_ij ]
!
! primitive Gaussians are thus normalized to 1
!
! input: shell number and start adress of the contraction in the list 
! of primitives
!
END_DOC
      implicit none
      integer ::  lval,nprimm,jp,ip,nshlf,ipst
      real*8 :: eps,pif,xnrm,xval,ccc,cip,dip,cjp,djp,exxx

!     write(6,*) ' entering normbf with ',nshlf,ipst

      xnrm=0.d0
! 0: s, 1: p,  ...
      lval=il(nshlf)
      nprimm=nprim(nshlf)
      eps=1.d-9
      xval=dble(lval)
      lval=lval+1
      pif=facc(lval)*(pi**1.5d0)
      pif=(2.d0**xval)/pif
      
      if (nprimm.eq.1) then
       coeff(ipst)=1.D0
      else
       ccc=0.d0
       do ip=ipst,ipst+nprimm-1
        cip=coeff(ip)
        dip=sqrt((2.d0*exx(ip))**(xval+1.5d0))
        do jp=ipst,ipst+nprimm-1
         exxx=1.d0/(exx(ip)+exx(jp))
         cjp=coeff(jp)
         djp=sqrt((2.d0*exx(jp))**(xval+1.5d0))
         ccc=ccc+cip*cjp*dip*djp*(exxx**(xval+1.5d0))
        end do
       end do
       do ip=ipst,ipst+nprimm-1
        coeff(ip)=coeff(ip)/sqrt(ccc)
       end do
      end if
      
    end subroutine normbf
