! -*- F90 -*-
BEGIN_PROVIDER [integer, nlmax ]
     nlmax=lmax
END_PROVIDER

BEGIN_PROVIDER [real*8, coeff_norm, (nprimt) ]
BEGIN_DOC
! renormalize individual coefficients for normalized primitives
END_DOC
       implicit none
       integer :: ishla,nprima,npsta,iia
       integer :: la,ia
       real*8 :: exa,xnorm

       do ishla=1,nshl
        la=il(ishla)
! the number and start address for the primitives
        nprima=nprim(ishla)
        npsta =npst (ishla)
        if (nprima.eq.1) then
         exa=exx(npsta)
         call norm_prim(exa,la,xnorm)
         coeff_norm(npsta)=xnorm
        else
         do iia=1,nprima
          ia=npsta+iia-1
          exa=exx(ia)
          call norm_prim(exa,la,xnorm)
          coeff_norm(ia)=coeff(ia)*xnorm
         end do
        end if
       end do
END_PROVIDER

BEGIN_PROVIDER [integer, ncmax]
     NCMAX=(NLMAX+1)*(NLMAX+2)/2
END_PROVIDER

 BEGIN_PROVIDER [real*8, OVERL, (NCMAX,NCMAX)]
&BEGIN_PROVIDER [real*8, DIPXC, (NCMAX,NCMAX)]
&BEGIN_PROVIDER [real*8, DIPYC, (NCMAX,NCMAX)]
&BEGIN_PROVIDER [real*8, DIPZC, (NCMAX,NCMAX)]
&BEGIN_PROVIDER [integer, numcar]
BEGIN_DOC
!  integrals in cartesians, for one block of two shells
END_DOC
END_PROVIDER

BEGIN_PROVIDER [real*8, thresh]
     thresh=1.D-11
END_PROVIDER

 BEGIN_PROVIDER [real*8, xa, (3)]
&BEGIN_PROVIDER [real*8, xb, (3)]
&BEGIN_PROVIDER [real*8, xab, (3)]
&BEGIN_PROVIDER [real*8, xp, (3)]
&BEGIN_PROVIDER [real*8, xpa, (3)]
&BEGIN_PROVIDER [real*8, xpb, (3)]
&BEGIN_PROVIDER [real*8, pi]
&BEGIN_PROVIDER [real*8, twopi]
     pi=2.D0*acos(0.D0)
     twopi=2.D0*pi
END_PROVIDER

 BEGIN_PROVIDER [integer, ncart, (0:nlmax) ]
&BEGIN_PROVIDER [real*8, carmul, (0:nlmax,ncmax) ]
      implicit none
      integer :: incr,l,i,j,m

      incr=1
      ncart(0)=1
      do l=1,nlmax
       incr=incr+1
       ncart(l)=ncart(l-1)+incr
      end do

      do i=0,nlmax
       do j=1,ncmax
        carmul(i,j)=0.D0
       end do
      end do

      do l=0,lmax
       do m=1,(l+1)*(l+2)/2
        carmul(l,m)=1.d0
       end do
      end do
      if (nlmax.gt.1) then
! proper normalization factors for cartesian d functions
       carmul(2,2)=sqrt(3.d0)
       carmul(2,4)=sqrt(3.d0)
       carmul(2,5)=sqrt(3.d0)
       if (nlmax.gt.2) then
! proper normalization factors for cartesian f functions
        carmul(3,2)=sqrt(5.d0)
        carmul(3,3)=sqrt(5.d0)
        carmul(3,5)=sqrt(5.d0)
        carmul(3,6)=sqrt(15.d0)
        carmul(3,7)=sqrt(5.d0)
        carmul(3,8)=sqrt(5.d0)
        carmul(3,9)=sqrt(5.d0)
       end if
      end if

END_PROVIDER

 BEGIN_PROVIDER [real*8, sao, (nbas,nbas)]
&BEGIN_PROVIDER [real*8, dipx, (nbas,nbas)]
&BEGIN_PROVIDER [real*8, dipy, (nbas,nbas)]
&BEGIN_PROVIDER [real*8, dipz, (nbas,nbas)]
BEGIN_DOC
! we calculate overlap and dipole integrals
! routine from the integral program, without kinetic and potential
!
END_DOC
      implicit none
      integer :: i,j,lab,ii,jj,l,l2,l1,indx2,indx1,ialph,ibeta,kk,k
      integer :: ia,ista,ishla,iata,la,nprima,npsta,iia
      integer :: ib,istb,ishlb,iatb,lb,nprimb,npstb,iib,imb,ima
      real*8 :: exa,exb,cca,ccb,coeffp,xmu,rab,p,emu,yyy,dipxh,dipyh,dipzh,xmult
      integer :: icarcar,m

      do i=1,nbas
       do j=1,nbas
        sao (i,j)=0.d0
        dipx(i,j)=0.d0
        dipy(i,j)=0.d0
        dipz(i,j)=0.d0
       end do
      end do
!
! the loop over atomic shells
!
! the real work
!
      ista=1
      do ishla=1,nshl
!       write(6,*) ' treating shell a = ',ishla
! the atom
       iata=icent(ishla)
! get the position
       do i=1,3
        xa(i)=pos(i,iata)
       end do
! the angular momentum
       la=il(ishla)
! the number and start address for the primitives
       nprima=nprim(ishla)
       npsta =npst (ishla)
!       write(6,*) ' la, nprima, npsta = ',la, nprima, npsta
       
       istb=ista
       do ishlb=ishla,nshl
!       write(6,*) '  treating shell b = ',ishlb
        iatb=icent(ishlb)
! get the angular momentum, the position, the primitives and the
! coefficients
        do i=1,3
         xb(i)=pos(i,iatb)
         xab(i)=xa(i)-xb(i)
        end do
        lb=il(ishlb)
! the number and start address for the primitives
        nprimb=nprim(ishlb)
        npstb =npst (ishlb)
        
! overlap and dipole integrals
!     
! set cartesian integrals to zero
!
        do i=1,ncart(la)
         do j=1,ncart(lb)
          overl(i,j)=0.d0
          dipxc(i,j)=0.d0
          dipyc(i,j)=0.d0
          dipzc(i,j)=0.d0
         end do
        end do
        
        lab=la+lb
        
        ia=npsta
        do iia=1,nprima
         exa=exx(ia)
         cca=coeff_norm(ia)
!         write(6,*) ' exa, cca ',exa,cca
         ia=ia+1
         
         ib=npstb
         do iib=1,nprimb
          exb=exx(ib)
          coeffp=cca*coeff_norm(ib)
          ib=ib+1
  
          p=exa+exb
          xmu=exa*exb/p
          rab=0.d0
          do i=1,3
           xp(i)=(exa*xa(i)+exb*xb(i))/p
           xpa(i)=-exb/p*xab(i)
           xpb(i)= exa/p*xab(i)
           rab=rab+xab(i)*xab(i)
           ss(i,0,0)=sqrt(pi/p)
           ss(i,1,0)=xpa(i)*ss(i,0,0)
           ss(i,0,1)=xpb(i)*ss(i,0,0)
          end do
          emu=exp(-xmu*rab)
          coeffp=coeffp*emu
          do i=1,3
           do l=2,la+lb+2
            jj=0
            if (l.le.la+2) then
             ss(i,l,0)=xpa(i)*ss(i,l-1,0)+0.5d0/p*dble(l-1)*ss(i,l-2,0)
            end if
            if (l.le.lb) then
             ss(i,0,l)=xpb(i)*ss(i,0,l-1)+0.5d0/p*dble(l-1)*ss(i,0,l-2)
            end if
! horizontal recurrence - we create all intermediates i+j=l
            do jj=1,l-1
             ii=l-jj
             if (ii.le.la+2.and.jj.le.lb) then
              yyy=dble(ii)*ss(i,ii-1,jj-1)
              if (jj.ge.2) then
               yyy=yyy+dble(jj-1)*ss(i,ii,jj-2)
              end if
              ss(i,ii,jj)=xpb(i)*ss(i,ii,jj-1)+0.5d0/p*yyy
             end if
            end do
           end do
          end do
!
          indx1=0
          do i=0,la
           do j=0,la-i
            indx1=indx1+1
            indx2=0
            do ii=0,lb
             dipxh=xp(1)*ss(1,i,ii)+0.5d0/p*(dble(i)*ss(1,i-1,ii) &
                  +dble(ii-1)*ss(1,i,ii-1)) 
             do jj=0,lb-ii
              indx2=indx2+1
              k =la-i-j
              kk=lb-ii-jj
              
              overl(indx1,indx2)+=coeffp*ss(1,i,ii)*ss(2,j,jj)*ss(3,k,kk)

              dipxh=xp(1)*ss(1,i,ii)+0.5d0/p*(dble(i)*ss(1,i-1,ii) &
                   +dble(ii)*ss(1,i,ii-1)) 
              dipyh=xp(2)*ss(2,j,jj)+0.5d0/p*(dble(j)*ss(2,j-1,jj) &
                   +dble(jj)*ss(2,j,jj-1)) 
              dipzh=xp(3)*ss(3,k,kk)+0.5d0/p*(dble(k)*ss(3,k-1,kk) &
                   +dble(kk)*ss(3,k,kk-1)) 

              dipxc(indx1,indx2)+= coeffp*dipxh*ss(2,j,jj)*ss(3,k,kk) 
              dipyc(indx1,indx2)+= coeffp*ss(1,i,ii)*dipyh*ss(3,k,kk) 
              dipzc(indx1,indx2)+= coeffp*ss(1,i,ii)*ss(2,j,jj)*dipzh

             end do
            end do
           end do
          end do
! close loops over primitives exa and exb
         end do
        end do
!
! if cartesian integrals are demanded we have all here
! we come from the recursion with xx, xy, xz, yy, yz, zz and so on
!
        do ima=1,ncart(la)
         ialph=ista+icarcar(la,ima)-1
         do imb=1,ncart(lb)
          ibeta=istb+icarcar(lb,imb)-1
          xmult=carmul(la,ima)*carmul(lb,imb)
          sao (ialph,ibeta)=overl(ima,imb)*xmult
          dipx(ialph,ibeta)=dipxc(ima,imb)*xmult
          dipy(ialph,ibeta)=dipyc(ima,imb)*xmult
          dipz(ialph,ibeta)=dipzc(ima,imb)*xmult
         end do
        end do
        istb=istb+ncart(lb)
!
! close loops over shells
!
       end do
       ista=ista+ncart(la)
      end do
END_PROVIDER

BEGIN_PROVIDER [real*8, ss, (3,-1:nlmax+2,-1:nlmax) ]
BEGIN_DOC
! temporary array for the recursion
END_DOC
      implicit none
      integer :: i,j,k
      do i=1,3
       do j=-1,nlmax+2
        do k=-1,nlmax
         ss(i,j,k)=0.d0
        end do
       end do
      end do
END_PROVIDER

      integer function icarcar(l,i)
        implicit none
        integer i,l,ll
        ll=(l+1)*(l+2)/2
        icarcar=ll-i+1
      end function icarcar

      subroutine norm_prim(expon,lval,facnor)
BEGIN_DOC
!
! here we normalize a primitive Gaussian function
!
! sqrt[
! l-dependent:
!         2^l*(2a)^(l+3/2)/(pi^3/2 (2l-1)!!)
! m-dependent: (not used here)
!          [(l-|m|)!/(l+|m|)!] * (2-delta_m0)
!       ]
!
END_DOC
        implicit none
        real*8 :: expon,facnor,xval,pif,ccc
        integer :: lval

        xval=dble(lval)
        pif=facc(lval+1)*(pi**1.5d0)
        pif=(2.d0**xval)/pif
        ccc=(2.d0*expon)**(xval+1.5d0)
        facnor=sqrt(pif*ccc)

      end subroutine norm_prim

 BEGIN_PROVIDER [real*8, FAC, (NLMAX+NLMAX+2)]
&BEGIN_PROVIDER [real*8, FACC, (NLMAX+1) ]
BEGIN_DOC
!
! the (2l-1)!! and the (l+|m|)!
! first the (2l-1)!!
!
END_DOC
      implicit none
      integer :: i,l
      real*8 :: xmlt

      fac(1)=1.d0
      fac(2)=1.d0
      do i=3,nlmax+nlmax+2
       xmlt=dble(i-1)
       fac(i)=xmlt*fac(i-1)
      end do
      facc(1)=1.d0
      do l=2,nlmax+1
       facc(l)=facc(l-1)*dble(l+l-3)
      end do
END_PROVIDER

 BEGIN_PROVIDER [real*8, smo, (nbas,nbas)]
&BEGIN_PROVIDER [real*8, sdiag, (nbas)]
&BEGIN_PROVIDER [real*8, halftr, (nbas,nbas)]
BEGIN_DOC
! the overlap matrix in molecular orbitales for testing the orthogonality 
! of the MOs 
END_DOC
       implicit none
       integer :: i,j,k
       real*8 :: sss

       do i=1,nbas
        do j=1,nbas
         smo(i,j)=0.D0
         halftr(i,j)=0.D0
        end do
        sdiag(i)=0.D0
       end do

       do i=1,nbas
        sss=0.D0
        do j=1,nbas
         do k=1,nbas
          sss+=cialpha(j,i)*cialpha(k,i)*sao(j,k)
         end do
        end do
        sdiag(i)=sss
       end do

       do i=1,nbas
        do j=1,nbas
         sss=0.D0
         do k=1,nbas
          sss+=sao(i,k)*cialpha(k,j)
         end do
         halftr(i,j)=sss
        end do
       end do

       do i=1,nbas
        do j=1,nbas
         sss=0.D0
         do k=1,nbas
          sss+=cialpha(k,i)*halftr(k,j)
         end do
         smo(i,j)=sss
        end do
       end do
END_PROVIDER


